fix problems with local anonymous declarations
[cparser] / ast_t.h
diff --git a/ast_t.h b/ast_t.h
index daceed6..1debd43 100644 (file)
--- a/ast_t.h
+++ b/ast_t.h
@@ -152,18 +152,18 @@ typedef enum {
        case EXPR_UNARY_ASSUME:                \
        case EXPR_UNARY_BITFIELD_EXTRACT:
 
-struct context_t {
-       declaration_t *declarations;  /**< List of declarations in this context. */
+struct scope_t {
+       declaration_t *declarations;  /**< List of declarations in this scope. */
 };
 
 struct expression_base_t {
        expression_kind_t   kind;
-       type_t             *datatype;
+       type_t             *type;
        source_position_t   source_position;
 };
 
 struct const_expression_t {
-       expression_base_t  expression;
+       expression_base_t  base;
        union {
                long long   int_value;
                long double float_value;
@@ -171,34 +171,34 @@ struct const_expression_t {
 };
 
 struct string_literal_expression_t {
-       expression_base_t  expression;
-       const char        *value;
+       expression_base_t  base;
+       string_t           value;
 };
 
 struct wide_string_literal_expression_t {
-       expression_base_t  expression;
+       expression_base_t  base;
        wide_string_t      value;
 };
 
 struct builtin_symbol_expression_t {
-       expression_base_t  expression;
+       expression_base_t  base;
        symbol_t          *symbol;
 };
 
 struct builtin_constant_expression_t {
-       expression_base_t  expression;
+       expression_base_t  base;
        expression_t      *value;
 };
 
 struct builtin_prefetch_expression_t {
-       expression_base_t  expression;
+       expression_base_t  base;
        expression_t      *adr;
        expression_t      *rw;
        expression_t      *locality;
 };
 
 struct reference_expression_t {
-       expression_base_t  expression;
+       expression_base_t  base;
        symbol_t          *symbol;
        declaration_t     *declaration;
 };
@@ -209,24 +209,24 @@ struct call_argument_t {
 };
 
 struct call_expression_t {
-       expression_base_t  expression;
+       expression_base_t  base;
        expression_t      *function;
        call_argument_t   *arguments;
 };
 
 struct unary_expression_t {
-       expression_base_t  expression;
+       expression_base_t  base;
        expression_t      *value;
 };
 
 struct binary_expression_t {
-       expression_base_t  expression;
+       expression_base_t  base;
        expression_t      *left;
        expression_t      *right;
 };
 
 struct select_expression_t {
-       expression_base_t  expression;
+       expression_base_t  base;
        expression_t      *compound;
        symbol_t          *symbol;
 
@@ -234,21 +234,16 @@ struct select_expression_t {
 };
 
 struct array_access_expression_t {
-       expression_base_t  expression;
+       expression_base_t  base;
        expression_t      *array_ref;
        expression_t      *index;
        bool               flipped; /* index/ref was written in a 5[a] way */
 };
 
-struct sizeof_expression_t {
-       expression_base_t  expression;
-       type_t            *type;
-       expression_t      *size_expression;
-};
-
-struct alignof_expression_t {
-       expression_base_t  expression;
+struct typeprop_expression_t {
+       expression_base_t  base;
        type_t            *type;
+       expression_t      *tp_expression;
 };
 
 struct designator_t {
@@ -258,36 +253,36 @@ struct designator_t {
 };
 
 struct offsetof_expression_t {
-       expression_base_t  expression;
+       expression_base_t  base;
        type_t            *type;
        designator_t      *designator;
 };
 
 struct va_start_expression_t {
-       expression_base_t  expression;
+       expression_base_t  base;
        expression_t      *ap;
        declaration_t     *parameter;
 };
 
 struct va_arg_expression_t {
-       expression_base_t  expression;
+       expression_base_t  base;
        expression_t      *ap;
 };
 
 struct conditional_expression_t {
-       expression_base_t  expression;
+       expression_base_t  base;
        expression_t      *condition;
        expression_t      *true_expression;
        expression_t      *false_expression;
 };
 
 struct statement_expression_t {
-       expression_base_t  expression;
+       expression_base_t  base;
        statement_t       *statement;
 };
 
 struct classify_type_expression_t {
-       expression_base_t  expression;
+       expression_base_t  base;
        expression_t      *type_expression;
 };
 
@@ -306,14 +301,13 @@ union expression_t {
        binary_expression_t              binary;
        select_expression_t              select;
        array_access_expression_t        array_access;
-       sizeof_expression_t              sizeofe;
+       typeprop_expression_t            typeprop;
        offsetof_expression_t            offsetofe;
        va_start_expression_t            va_starte;
        va_arg_expression_t              va_arge;
        conditional_expression_t         conditional;
        statement_expression_t           statement;
        classify_type_expression_t       classify_type;
-       alignof_expression_t             alignofe;
 };
 
 typedef enum {
@@ -360,8 +354,8 @@ struct initializer_list_t {
 };
 
 struct initializer_string_t {
-       initializer_base_t  initializer;
-       const char         *string;
+       initializer_base_t initializer;
+       string_t           string;
 };
 
 struct initializer_wide_string_t {
@@ -397,6 +391,7 @@ struct declaration_t {
        decl_modifiers_t    modifiers;
        unsigned int        address_taken : 1;
        unsigned int        is_inline     : 1;
+       unsigned int        used          : 1;  /**< Set if the declaration is used. */
        type_t             *type;
        symbol_t           *symbol;
        source_position_t   source_position;
@@ -406,10 +401,10 @@ struct declaration_t {
                initializer_t  *initializer;
                expression_t   *enum_value;
        } init;
-       context_t           context;
-       context_t          *parent_context;
+       scope_t             scope;
+       scope_t            *parent_scope;
 
/** next declaration in a context */
        declaration_t      *next;
        /** next declaration with same symbol */
        declaration_t      *symbol_next;
@@ -450,96 +445,97 @@ struct statement_base_t {
 };
 
 struct return_statement_t {
-       statement_base_t  statement;
-       expression_t     *return_value;
+       statement_base_t  base;
+       expression_t     *value;
 };
 
 struct compound_statement_t {
-       statement_base_t  statement;
+       statement_base_t  base;
        statement_t      *statements;
-       context_t         context;
+       scope_t           scope;
 };
 
 struct declaration_statement_t {
-       statement_base_t  statement;
+       statement_base_t  base;
        declaration_t    *declarations_begin;
        declaration_t    *declarations_end;
 };
 
 struct if_statement_t {
-       statement_base_t  statement;
+       statement_base_t  base;
        expression_t     *condition;
        statement_t      *true_statement;
        statement_t      *false_statement;
 };
 
 struct switch_statement_t {
-       statement_base_t       statement;
+       statement_base_t       base;
        expression_t           *expression;
        statement_t            *body;
        case_label_statement_t *first_case, *last_case;
 };
 
 struct goto_statement_t {
-       statement_base_t  statement;
-       declaration_t    *label;
-       goto_statement_t *next; /**< link all goto statements in a function */
+       statement_base_t  base;
+       declaration_t    *label;     /**< The destination label. */
+       goto_statement_t *next;      /**< links all goto statements of a function */
 };
 
 struct case_label_statement_t {
-       statement_base_t        statement;
+       statement_base_t        base;
        expression_t           *expression;
-       statement_t            *label_statement;
-       case_label_statement_t *next; /**< link to the next case label in the switch */
+       statement_t            *statement;
+       case_label_statement_t *next; /**< link to the next case label in switch */
 };
 
 struct label_statement_t {
-       statement_base_t  statement;
-       declaration_t    *label;
-       statement_t      *label_statement;
+       statement_base_t   base;
+       declaration_t     *label;
+       statement_t       *statement;
+       label_statement_t *next;    /**< links all label statements of a function */
 };
 
 struct expression_statement_t {
-       statement_base_t  statement;
+       statement_base_t  base;
        expression_t     *expression;
 };
 
 struct while_statement_t {
-       statement_base_t  statement;
+       statement_base_t  base;
        expression_t     *condition;
        statement_t      *body;
 };
 
 struct do_while_statement_t {
-       statement_base_t  statement;
+       statement_base_t  base;
        expression_t     *condition;
        statement_t      *body;
 };
 
 struct for_statement_t {
-       statement_base_t  statement;
+       statement_base_t  base;
        expression_t     *initialisation;
        expression_t     *condition;
        expression_t     *step;
        statement_t      *body;
-       context_t         context;
+       scope_t           scope;
 };
 
 struct asm_constraint_t {
-       const char       *constraints;
+       string_t          constraints;
        expression_t     *expression;
        symbol_t         *symbol;
        asm_constraint_t *next;
 };
 
 struct asm_clobber_t {
-       const char    *clobber;
+       string_t       clobber;
        asm_clobber_t *next;
 };
 
 struct asm_statement_t {
-       statement_base_t  statement;
-       const char       *asm_text;
+       statement_base_t  base;
+       string_t          asm_text;
        asm_constraint_t *inputs;
        asm_constraint_t *outputs;
        asm_clobber_t    *clobbers;
@@ -565,7 +561,7 @@ union statement_t {
 };
 
 struct translation_unit_t {
-       context_t context;
+       scope_t scope;
 };
 
 static inline
@@ -406,10 +401,10 @@ struct declaration_t {
 
-       /** next declaration in a context */
+       /** next declaration in a scope */
        declaration_t      *next;
        /** next declaration with same symbol */